package wsagency;

import data.*;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.*;
import javax.rmi.*;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeader;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
 
public class LocationInformationService extends HttpServlet
{
    static final String jobJNDI = "java:comp/env/ejb/JobLocal";
    static final String applicantJNDI = "java:comp/env/ejb/ApplicantLocal";

	public void doPost(HttpServletRequest request, HttpServletResponse response)
    					throws ServletException, IOException
	{
		log("LocationInformationService: Called");
    
		try
		{
			MimeHeaders mimeHeaders = getMimeHeaders(request);

			SOAPMessage soapRequest = extractSoapMessage(mimeHeaders, request);

			String requestType = ServerMarshaler.getRequestTypeFromMessage(soapRequest);
			String location = ServerMarshaler.getLocationFromMessage(soapRequest);

    		if (requestType == null || requestType.length() == 0)
			{
				log("LocationInformationService: no request type provided");
				return;
			}

    		if (location == null || location.length() == 0)
			{
				log("LocationInformationService: no location provided");
				return;
			}

        	InitialContext ic = new InitialContext();

			SOAPMessage soapResponse = createSoapMessage();

			if (requestType.equalsIgnoreCase("jobsAtLocation"))
			{
				String[] jobs = null;

        		Object lookup = ic.lookup(jobJNDI);

        		JobLocalHome jobHome = (JobLocalHome)PortableRemoteObject.narrow(lookup, JobLocalHome.class);

        		Collection col = jobHome.findByLocation(location);
        		Iterator it = col.iterator();

				jobs = new String[col.size()];
        		for (int i = 0; i < col.size(); ++i)
				{
            		JobLocal job = (JobLocal)it.next();
            		jobs[i] = job.getCustomer() + "/" + job.getRef();
        		}

				ServerMarshaler.addJobsToMessage(jobs, soapResponse);
			}
			else if (requestType.equalsIgnoreCase("applicantsAtLocation"))
			{
				String[] applicants = null;

        		Object lookup = ic.lookup(applicantJNDI);

        		ApplicantLocalHome applicantHome = (ApplicantLocalHome)PortableRemoteObject.narrow(lookup, ApplicantLocalHome.class);

        		Collection col = applicantHome.findByLocation(location);
        		Iterator it = col.iterator();

				applicants = new String[col.size()];
        		for (int i = 0; i < col.size(); ++i)
				{
            		ApplicantLocal applicant = (ApplicantLocal)it.next();
            		applicants[i] = applicant.getLogin();
        		}

				ServerMarshaler.addApplicantsToMessage(applicants, soapResponse);
			}

			sendSoapMessage(response, soapResponse);
		}
		catch (Exception ex)
		{
			log("Exception: ", ex);
			throw new ServletException("Exception handling SOAP message: " + ex.getMessage() + ", " + ex);
		}
	}

	private void sendSoapMessage(HttpServletResponse response, SOAPMessage soapMessage)
										throws IOException, SOAPException
	{
       	response.setStatus(HttpServletResponse.SC_OK);

		response.setHeader("content-type", "text/xml; charset=\"utf-8\"");

       	OutputStream outStream = response.getOutputStream();
       	soapMessage.writeTo(outStream);

       	outStream.flush();
	}

	private SOAPMessage extractSoapMessage(MimeHeaders mimeHeaders, HttpServletRequest request)
							throws SOAPException, IOException
	{
        	InputStream inStream = request.getInputStream();

        	MessageFactory factory = getMessageFactory();
        	SOAPMessage message = factory.createMessage(mimeHeaders, inStream);

		return message;
	}

	private SOAPMessage createSoapMessage() throws SOAPException, IOException
	{
        	MessageFactory factory = getMessageFactory();
        	SOAPMessage message = factory.createMessage();

		return message;
	}

	private MimeHeaders getMimeHeaders(HttpServletRequest request)
	{
 		Enumeration httpHeaders = request.getHeaderNames();
    		MimeHeaders mimeHeaders = new MimeHeaders();

    		while (httpHeaders.hasMoreElements())
		{
        		String httpHeaderName = (String)httpHeaders.nextElement();
        		String httpHeaderValue = request.getHeader(httpHeaderName);

			// log("MimeHeader: (" + httpHeaderName + ", " + httpHeaderValue + ")");

        		StringTokenizer httpHeaderValues = new StringTokenizer(httpHeaderValue, ",");
        		while (httpHeaderValues.hasMoreTokens())
			{
				String headerValue = httpHeaderValues.nextToken().trim();
        			mimeHeaders.addHeader(httpHeaderName, headerValue);
			}
    		}

    		return mimeHeaders;
	}

	private MessageFactory messageFactory = null;
	public MessageFactory getMessageFactory() throws SOAPException
	{
		if (messageFactory == null)
		{
			messageFactory = MessageFactory.newInstance();
		}
		return messageFactory;
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
    					throws ServletException, IOException
	{
		PrintWriter writer = response.getWriter();
		writer.println("<html>");
		writer.println("<body>");
		writer.println("<p>");
		writer.println("This servlet is a message-based SOAP endpoint. Please send SOAP messages using HTTP POST.");
		writer.println("</p>");
		writer.println("</body>");
		writer.println("</html>");
		writer.flush();
		writer.close();
	}
}
