package wsagency;

import java.util.Iterator;

import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;

public class ServerMarshaler
{
  static final String nsAgency = "http://J2EE21/agency";
  static final String prefixAgency = "agency";
	
  public static String getRequestTypeFromMessage(SOAPMessage soapMessage) throws SOAPException
  {
    SOAPEnvelope soapEnvelope = soapMessage.getSOAPPart().getEnvelope();
    SOAPBody soapBody = soapEnvelope.getBody();

    // Retrieve the request element from under the body
    Name requestName = soapEnvelope.createName("request", prefixAgency, nsAgency);
    SOAPElement soapRequest = getChildElementByName(requestName, soapBody);

    // Retrieve the type element from under the request
    Name typeName = soapEnvelope.createName("type", prefixAgency, nsAgency);
    SOAPElement soapType = getChildElementByName(typeName, soapRequest);
    String type = soapType.getValue();

    return type;
  }

  public static String getLocationFromMessage(SOAPMessage soapMessage) throws SOAPException
  {
    SOAPEnvelope soapEnvelope = soapMessage.getSOAPPart().getEnvelope();
    SOAPBody soapBody = soapEnvelope.getBody();

    // Retrieve the request element from under the body
    Name requestName = soapEnvelope.createName("request", prefixAgency, nsAgency);
    SOAPElement soapRequest = getChildElementByName(requestName, soapBody);

    // Retrieve the location element from under the request
    Name locationName = soapEnvelope.createName("location", prefixAgency, nsAgency);
    SOAPElement soapLocation = getChildElementByName(locationName, soapRequest);
    String location = soapLocation.getValue();

    return location;
  }

  public static void addJobsToMessage(String[] jobs, SOAPMessage soapMessage) throws SOAPException
  {
    SOAPEnvelope soapEnvelope = soapMessage.getSOAPPart().getEnvelope();
    SOAPBody soapBody = soapEnvelope.getBody();

    // Create a response element under the body
    Name responseName = soapEnvelope.createName("response", prefixAgency, nsAgency);
    SOAPBodyElement soapResponse = soapBody.addBodyElement(responseName);

    // Indicate how many job elements there are in the response
    Name numJobsName = soapEnvelope.createName("numJobs", prefixAgency, nsAgency);
    SOAPElement soapNumJobs = soapResponse.addChildElement(numJobsName);
    soapNumJobs.addTextNode("" + jobs.length);

    // Create a set of job elements under the response
    Name jobsName = soapEnvelope.createName("jobs", prefixAgency, nsAgency);
    SOAPElement soapJobs = soapResponse.addChildElement(jobsName);
    
    Name jobName = soapEnvelope.createName("job", prefixAgency, nsAgency);

    for (int i = 0; i < jobs.length; i++)
    {
      SOAPElement soapJob= soapJobs.addChildElement(jobName);
      soapJob.addTextNode(jobs[i]);
    }
  }

  public static void addApplicantsToMessage(String[] applicants, SOAPMessage soapMessage) throws SOAPException
  {
    SOAPEnvelope soapEnvelope = soapMessage.getSOAPPart().getEnvelope();
    SOAPBody soapBody = soapEnvelope.getBody();

    // Create a response element under the body
    Name responseName = soapEnvelope.createName("response", prefixAgency, nsAgency);
    SOAPBodyElement soapResponse = soapBody.addBodyElement(responseName);

    // Indicate how many applicant elements there are in the response
    Name numApplicantsName = soapEnvelope.createName("numApplicants", prefixAgency, nsAgency);
    SOAPElement soapNumApplicants = soapResponse.addChildElement(numApplicantsName);
    soapNumApplicants.addTextNode("" + applicants.length);

    // Create a set of applicant elements under the response
    Name applicantsName = soapEnvelope.createName("applicants", prefixAgency, nsAgency);
    SOAPElement soapApplicants = soapResponse.addChildElement(applicantsName);
    
    Name applicantName = soapEnvelope.createName("applicant", prefixAgency, nsAgency);

    for (int i = 0; i < applicants.length; i++)
    {
      SOAPElement soapApplicant = soapApplicants.addChildElement(applicantName);
      soapApplicant.addTextNode(applicants[i]);
    }
  }

  private static SOAPElement getChildElementByName(Name name, SOAPElement element)
	  						throws SOAPException
  {
    Iterator childIterator = element.getChildElements(name);
    
    if (childIterator.hasNext())
    {
      SOAPElement childElement = (SOAPElement)childIterator.next();

      return childElement;
    }
    else
    {
      throw new SOAPException("element " + name.getQualifiedName() + " is missing");
    }
  }

  private static boolean debug = false;
  private static void debug(String msg)
  {
	  if (debug)
	  {
	  	System.out.println(msg);
	  }
  }
}
