package client;

import java.util.*;
import java.net.PasswordAuthentication;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.JAXRException;
import javax.xml.registry.RegistryService;

import javax.xml.registry.infomodel.EmailAddress;
import javax.xml.registry.infomodel.InternationalString;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.PersonName;
import javax.xml.registry.infomodel.TelephoneNumber;
import javax.xml.registry.infomodel.User;

public class JAXROrgLister
{
  public static void main(String[] args)
  {
	  try
	  {
		  String queryString = "Wid";

		JAXROrgLister lister = new JAXROrgLister();

		lister.setAuthenticationInformation("testuser", "testuser");

		Collection organizations = lister.listOrganizationsByName(queryString);

		System.out.println("There are " + organizations.size() + " organizations that begin with " + queryString);

		Iterator iterator = organizations.iterator();
		while (iterator.hasNext())
		{
			Organization organization = (Organization)iterator.next();

			System.out.println("Organization name:\t\t" + organization.getName().getValue());
			System.out.println("Organization description:\t" + organization.getDescription().getValue());

			User primaryContact = organization.getPrimaryContact();
			System.out.println("Primary contact:\t\t" + primaryContact.getPersonName().getFullName());

			Collection phoneNumbers = primaryContact.getTelephoneNumbers(null);
			Iterator phoneIterator = phoneNumbers.iterator();
			if (phoneIterator.hasNext())
			{
				TelephoneNumber number = (TelephoneNumber)phoneIterator.next();
				System.out.println("Primary contact phone:\t" + number.getNumber());
			}

			Collection emailAddresses = primaryContact.getEmailAddresses();
			Iterator emailIterator = emailAddresses.iterator();
			if (emailIterator.hasNext())
			{
				EmailAddress address = (EmailAddress)emailIterator.next();
				System.out.println("Primary contact email:\t" + address.getAddress());
			}
		}
	  }
	  catch (Exception ex)
	  {
		  System.err.println("Exception: " + ex + ", " + ex.getMessage());
		  ex.printStackTrace(System.out);
	  }
  }

  private Collection listOrganizationsByName(String queryString)
	  					throws NamingException, JAXRException
  {
	Collection findQualifiers = new ArrayList();
	findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);

	Collection namePatterns = new ArrayList();
	namePatterns.add(queryString);

	BusinessQueryManager queryManager = getQueryManager();

	BulkResponse bulkResponse = queryManager.findOrganizations(findQualifiers,
								   namePatterns,
								   null,
								   null,
								   null,
								   null);
	Collection organizations = bulkResponse.getCollection();

	return organizations;
  }

  private RegistryService m_registryService = null;
  private RegistryService getRegistryService()
	  			throws NamingException, JAXRException
  {
	  if (m_registryService == null)
	  {
		m_registryService = getConnection().getRegistryService();
	  }
	  return m_registryService;
  }

  private BusinessLifeCycleManager m_lifecycleManager = null;
  private BusinessLifeCycleManager getLifeCycleManager()
	  			throws NamingException, JAXRException
  {
	if (m_lifecycleManager == null)
	{
		m_lifecycleManager = getRegistryService().getBusinessLifeCycleManager(); 
	}
	return m_lifecycleManager;
  }  

  private BusinessQueryManager m_queryManager = null;
  private BusinessQueryManager getQueryManager()
	  			throws NamingException, JAXRException
  {
	if (m_queryManager == null)
	{
		m_queryManager = getRegistryService().getBusinessQueryManager(); 
	}
	return m_queryManager;
  }
  
  private void setAuthenticationInformation(String user, String password)
	  				throws NamingException, JAXRException
  {
	  Connection connection = getConnection();

	Set credentials = new HashSet();

	char[] passwordArray = password.toCharArray();

	PasswordAuthentication pwdAuthentication = new PasswordAuthentication(user, 
        								      passwordArray);

	credentials.add(pwdAuthentication);

	connection.setCredentials(credentials); 
  }

  private Connection m_connection = null;
  private Connection getConnection()
	  		throws NamingException, JAXRException
  {
	  if (m_connection == null)
	  {
		Properties riRegistryProperties = new Properties();
		// riRegistryProperties.setProperty("javax.xml.registry.queryManagerURL",
    	//	"http://localhost:8000/RegistryServer/RegistryServerServlet");
		riRegistryProperties.setProperty("javax.xml.registry.queryManagerURL",
    		"http://localhost:8000/RegistryServer/");
		riRegistryProperties.setProperty("javax.xml.registry.lifeCycleManagerURL",
    		"http://localhost:8000/RegistryServer/");

		Context context = new InitialContext();

   		ConnectionFactory factory = (ConnectionFactory)context.lookup("java:comp/env/eis/JAXR"); 

        factory.setProperties(riRegistryProperties);

		m_connection = factory.createConnection();
	  }
	  return m_connection;
  }
}
