package client;

import java.util.*;
import java.net.PasswordAuthentication;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.JAXRException;
import javax.xml.registry.RegistryService;

import javax.xml.registry.infomodel.EmailAddress;
import javax.xml.registry.infomodel.InternationalString;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.PersonName;
import javax.xml.registry.infomodel.TelephoneNumber;
import javax.xml.registry.infomodel.User;

public class JAXROrgPublisher
{
  public static void main(String[] args)
  {
	  try
	  {
		JAXROrgPublisher publisher = new JAXROrgPublisher();

		publisher.setAuthenticationInformation("testuser", "testuser");

		publisher.registerOrganization("Widgets Inc.",
				     "Manufacturers of the finest silver-plated widgets",
				     "Fred Bloggs",
				     "(+44) 999 7826",
				     "fred@widgets-i.co.uk");

		publisher.commitData();
		
	  }
	  catch (Exception ex)
	  {
		  System.err.println("Exception: " + ex + ", " + ex.getMessage());
		  ex.printStackTrace(System.out);
	  }
  }

  // Start simply and then maybe add classification and/or
  // services/bindings etc. if time allows
  private void registerOrganization(String organizationName,
		  		    String organizationDescription,
				    String contactName,
				    String contactNumber,
				    String contactEmail)
	  					throws NamingException, JAXRException
  {
	BusinessLifeCycleManager lifecycleManager = getLifeCycleManager();

	Organization regOrganization = lifecycleManager.createOrganization(organizationName);
	InternationalString regDescription = lifecycleManager.createInternationalString(organizationDescription);
	regOrganization.setDescription(regDescription);

	User regPrimaryContact = lifecycleManager.createUser();
	PersonName regContactName = lifecycleManager.createPersonName(contactName);
	regPrimaryContact.setPersonName(regContactName);

	TelephoneNumber regContactNumber = lifecycleManager.createTelephoneNumber();
	regContactNumber.setNumber(contactNumber);
	Collection numbers = new ArrayList();
	numbers.add(regContactNumber);
	regPrimaryContact.setTelephoneNumbers(numbers);

	EmailAddress emailAddress = lifecycleManager.createEmailAddress(contactEmail);
	Collection addresses = new ArrayList();
	addresses.add(emailAddress);
	regPrimaryContact.setEmailAddresses(addresses);

	regOrganization.setPrimaryContact(regPrimaryContact); 

/*
classification.scheme=ntis-gov:naics
classification.name=Snack and Nonalcoholic Beverage Bars
classification.value=722213
service.name=My Service Name
service.description=My Service Description
svcbinding.description=My Service Binding Description
svcbinding.accessURI=http://TheCoffeeBreak.com:8080/sb/
 */

	pending.add(regOrganization);
  }

  private Collection pending = new ArrayList();

  private boolean commitData() throws NamingException, JAXRException
  {
	BulkResponse bulkResponse = getLifeCycleManager().saveOrganizations(pending);
	Collection exceptions = bulkResponse.getExceptions();

	if (exceptions == null)
	{
    		System.out.println("Saved organizations");
		pending.clear();
		return true;
    	}
    	else
    	{
	    	System.err.println("Got exceptions when trying to save organizations");
		return false;
    	}

  }

  private RegistryService m_registryService = null;
  private RegistryService getRegistryService()
	  			throws NamingException, JAXRException
  {
	  if (m_registryService == null)
	  {
		m_registryService = getConnection().getRegistryService();
	  }
	  return m_registryService;
  }

  private BusinessLifeCycleManager m_lifecycleManager = null;
  private BusinessLifeCycleManager getLifeCycleManager()
	  			throws NamingException, JAXRException
  {
	if (m_lifecycleManager == null)
	{
		m_lifecycleManager = getRegistryService().getBusinessLifeCycleManager(); 
	}
	return m_lifecycleManager;
  }
  
  private void setAuthenticationInformation(String user, String password)
	  				throws NamingException, JAXRException
  {
	  Connection connection = getConnection();

	Set credentials = new HashSet();

	char[] passwordArray = password.toCharArray();

	PasswordAuthentication pwdAuthentication = new PasswordAuthentication(user, 
        								      passwordArray);

	credentials.add(pwdAuthentication);

	connection.setCredentials(credentials); 
  }

  private Connection m_connection = null;
  private Connection getConnection()
	  		throws NamingException, JAXRException
  {
	  if (m_connection == null)
	  {
		Context context = new InitialContext();
    		ConnectionFactory factory = (ConnectionFactory)context.lookup("eis/JAXR"); 

		Properties riRegistryProperties = new Properties();
		riRegistryProperties.setProperty("javax.xml.registry.queryManagerURL",
    		"http://localhost:8000/RegistryServer/RegistryServerServlet");
		riRegistryProperties.setProperty("javax.xml.registry.lifeCycleManagerURL",
    		"http://localhost:8000/RegistryServer/RegistryServerServlet");

		factory.setProperties(riRegistryProperties);

		m_connection = factory.createConnection();
	  }
	  return m_connection;
  }
}
