package client;

import java.util.Iterator;

import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;

public class ClientMarshaler
{
  static final String nsAgency = "http://J2EE21/agency";
  static final String prefixAgency = "agency";
	
  public static void addRequestToMessage(String type, String location, SOAPMessage soapMessage) throws SOAPException
  {
    SOAPEnvelope soapEnvelope = soapMessage.getSOAPPart().getEnvelope();
    SOAPBody soapBody = soapEnvelope.getBody();
    
    // Create a request element under the body
    Name requestName = soapEnvelope.createName("request", prefixAgency, nsAgency);
    SOAPBodyElement soapRequest = soapBody.addBodyElement(requestName);

    // Create a type element under the agencyRequest
    Name typeName = soapEnvelope.createName("type", prefixAgency, nsAgency);
    SOAPElement soapType = soapRequest.addChildElement(typeName);
    soapType.addTextNode(type);
    
    // Create a location element under the agencyRequest
    Name locationName = soapEnvelope.createName("location", prefixAgency, nsAgency);
    SOAPElement soapLocation = soapRequest.addChildElement(locationName);
    soapLocation.addTextNode(location);
  }
  
  public static String[] getJobsFromMessage(SOAPMessage soapMessage) throws SOAPException
  {
    SOAPEnvelope soapEnvelope = soapMessage.getSOAPPart().getEnvelope();
    SOAPBody soapBody = soapEnvelope.getBody();

    // Retrieve the response element from under the body
    Name responseName = soapEnvelope.createName("response", prefixAgency, nsAgency);
    SOAPElement soapResponse = getChildElementByName(responseName, soapBody);

    // Retrieve the numJobs element from under the response
    Name numJobsName = soapEnvelope.createName("numJobs", prefixAgency, nsAgency);
    SOAPElement soapNumJobs = getChildElementByName(numJobsName, soapResponse);
    
    int numJobs = Integer.parseInt(soapNumJobs.getValue());
    String[] jobs = new String[numJobs];

    // Retrieve the jobs element from under the response
    Name jobsName = soapEnvelope.createName("jobs", prefixAgency, nsAgency);
    SOAPElement soapJobs = getChildElementByName(jobsName, soapResponse);

    // Retrieve the jobs and put them in an array
    Iterator jobIterator = soapJobs.getChildElements();
      
    for (int i = 0; i < numJobs && jobIterator.hasNext(); i++)
    {
      SOAPElement jobElement = (SOAPElement)jobIterator.next();
      jobs[i] = jobElement.getValue();
    }

    return jobs;
  }

  private static SOAPElement getChildElementByName(Name name, SOAPElement element)
	  						throws SOAPException
  {
    Iterator childIterator = element.getChildElements(name);
    
    if (childIterator.hasNext())
    {
      SOAPElement childElement = (SOAPElement)childIterator.next();

      return childElement;
    }
    else
    {
      throw new SOAPException("element " + name.getQualifiedName() + " is missing");
    }
  }

  private static boolean debug = false;
  private static void debug(String msg)
  {
	  if (debug)
	  {
	  	System.out.println(msg);
	  }
  }
}
